@@ -36,8 +36,10 @@
 #include <spa/node/node.h>
 #include <spa/utils/keys.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/support/loop.h>
 #include <spa/support/plugin.h>
+#include <spa/support/i18n.h>
 #include <spa/monitor/device.h>
 #include <spa/monitor/utils.h>
 #include <spa/monitor/event.h>
@@ -48,6 +50,8 @@
 
 #include "acp/acp.h"
 
+extern struct spa_i18n *acp_i18n;
+
 #define NAME  "alsa-device"
 
 #define MAX_POLL	16
@@ -144,12 +148,12 @@ static int emit_node(struct impl *this, struct acp_device *dev)
 	struct spa_dict_item *items;
 	const struct acp_dict_item *it;
 	uint32_t n_items, i;
-	char device_name[128], path[180], channels[16], ch[12];
-	char card_id[16], *p;
+	char device_name[128], path[180], channels[16], ch[12], routes[16];
+	char card_index[16], *p;
 	char positions[SPA_AUDIO_MAX_CHANNELS * 12];
 	struct spa_device_object_info info;
 	struct acp_card *card = this->card;
-	const char *stream, *devstr;;
+	const char *stream, *devstr;
 
 	info = SPA_DEVICE_OBJECT_INFO_INIT();
 	info.type = SPA_TYPE_INTERFACE_Node;
@@ -164,10 +168,10 @@ static int emit_node(struct impl *this, struct acp_device *dev)
 
 	info.change_mask = SPA_DEVICE_OBJECT_CHANGE_MASK_PROPS;
 
-	n_items = dev->props.n_items + 6;
-	items = alloca(n_items * sizeof(*items));
+	items = alloca((dev->props.n_items + 8) * sizeof(*items));
+	n_items = 0;
 
-	snprintf(card_id, sizeof(card), "%d", card->index);
+	snprintf(card_index, sizeof(card_index), "%d", card->index);
 
 	devstr = dev->device_strings[0];
 	p = strstr(devstr, "%f");
@@ -178,23 +182,27 @@ static int emit_node(struct impl *this, struct acp_device *dev)
 	} else {
 		snprintf(device_name, sizeof(device_name), "%s", devstr);
 	}
-	snprintf(path, sizeof(path), "alsa:pcm:%s:%s:%s", card_id, device_name, stream);
-	items[0] = SPA_DICT_ITEM_INIT(SPA_KEY_OBJECT_PATH,	       path);
-	items[1] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PATH,           device_name);
-	items[2] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PCM_CARD,       card_id);
-	items[3] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PCM_STREAM,     stream);
+	snprintf(path, sizeof(path), "alsa:pcm:%s:%s:%s", card_index, device_name, stream);
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_OBJECT_PATH, path);
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PATH, device_name);
+	if (dev->flags & ACP_DEVICE_UCM_DEVICE)
+		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_OPEN_UCM, "true");
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PCM_CARD, card_index);
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_API_ALSA_PCM_STREAM, stream);
 
 	snprintf(channels, sizeof(channels), "%d", dev->format.channels);
-	items[4] = SPA_DICT_ITEM_INIT(SPA_KEY_AUDIO_CHANNELS, channels);
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_AUDIO_CHANNELS, channels);
 
 	p = positions;
 	for (i = 0; i < dev->format.channels; i++) {
 		p += snprintf(p, 12, "%s%s", i == 0 ? "" : ",",
 				acp_channel_str(ch, sizeof(ch), dev->format.map[i]));
 	}
+	items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_AUDIO_POSITION, positions);
+
+	snprintf(routes, sizeof(routes), "%d", dev->n_ports);
+	items[n_items++] = SPA_DICT_ITEM_INIT("device.routes", routes);
 
-	items[5] = SPA_DICT_ITEM_INIT(SPA_KEY_AUDIO_POSITION, positions);
-	n_items = 6;
 	acp_dict_for_each(it, &dev->props)
 		items[n_items++] = SPA_DICT_ITEM_INIT(it->key, it->value);
 
@@ -213,6 +221,7 @@ static int emit_info(struct impl *this, bool full)
 	const struct acp_dict_item *it;
 	struct acp_card *card = this->card;
 	char path[128];
+	uint64_t old = full ? this->info.change_mask : 0;
 
 	if (full)
 		this->info.change_mask = this->info_all;
@@ -241,7 +250,7 @@ static int emit_info(struct impl *this, bool full)
 			}
 		}
 		spa_device_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 	return err;
 }
@@ -400,11 +409,14 @@ static struct spa_pod *build_route(struct spa_pod_builder *b, uint32_t id,
 	if (dev != NULL) {
 		uint32_t channels = dev->format.channels;
 		float volumes[channels];
+		float soft_volumes[channels];
 		bool mute;
 
 		acp_device_get_mute(dev, &mute);
 		spa_zero(volumes);
+		spa_zero(soft_volumes);
 		acp_device_get_volume(dev, volumes, channels);
+		acp_device_get_soft_volume(dev, soft_volumes, channels);
 
 		spa_pod_builder_prop(b, SPA_PARAM_ROUTE_device, 0);
 		spa_pod_builder_int(b, dev->index);
@@ -432,6 +444,10 @@ static struct spa_pod *build_route(struct spa_pod_builder *b, uint32_t id,
 		spa_pod_builder_array(b, sizeof(uint32_t), SPA_TYPE_Id,
 				channels, dev->format.map);
 
+		spa_pod_builder_prop(b, SPA_PROP_softVolumes, 0);
+		spa_pod_builder_array(b, sizeof(float), SPA_TYPE_Float,
+				channels, soft_volumes);
+
 		spa_pod_builder_pop(b, &f[1]);
 	}
 	spa_pod_builder_prop(b, SPA_PARAM_ROUTE_devices, 0);
@@ -555,7 +571,7 @@ static int apply_device_props(struct impl *this, struct acp_device *dev, struct
 	int changed = 0;
 	float volumes[ACP_MAX_CHANNELS];
 	uint32_t channels[ACP_MAX_CHANNELS];
-	uint32_t n_volumes = 0, n_channels = 0;
+	uint32_t n_volumes = 0;
 
 	if (!spa_pod_is_object_type(props, SPA_TYPE_OBJECT_Props))
 		return -EINVAL;
@@ -581,8 +597,8 @@ static int apply_device_props(struct impl *this, struct acp_device *dev, struct
 			}
 			break;
 		case SPA_PROP_channelMap:
-			if ((n_channels = spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
-					channels, ACP_MAX_CHANNELS)) > 0) {
+			if (spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
+					channels, ACP_MAX_CHANNELS) > 0) {
 				changed++;
 			}
 			break;
@@ -621,7 +637,7 @@ static int impl_set_param(void *object,
 			return res;
 		}
 
-		res = acp_card_set_profile(this->card, id, save ? ACP_PROFILE_SAVE : 0);
+		acp_card_set_profile(this->card, id, save ? ACP_PROFILE_SAVE : 0);
 		emit_info(this, false);
 		break;
 	}
@@ -649,7 +665,7 @@ static int impl_set_param(void *object,
 			return -EINVAL;
 
 		dev = this->card->devices[device];
-		res = acp_device_set_port(dev, id, save ? ACP_PORT_SAVE : 0);
+		acp_device_set_port(dev, id, save ? ACP_PORT_SAVE : 0);
 		if (props)
 			apply_device_props(this, dev, props);
 		emit_info(this, false);
@@ -778,31 +794,24 @@ static void card_port_available(void *data, uint32_t index,
 }
 
 static void on_volume_changed(void *data, struct acp_device *dev)
-{
-	struct impl *this = data;
-	spa_log_info(this->log, "device %s volume changed", dev->name);
-	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[IDX_Route].user++;
-}
-
-static void on_mute_changed(void *data, struct acp_device *dev)
-{
-	struct impl *this = data;
-	spa_log_info(this->log, "device %s mute changed", dev->name);
-	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-	this->params[IDX_Route].user++;
-}
-
-static void on_set_soft_volume(void *data, struct acp_device *dev,
-		const float *volume, uint32_t n_volume)
 {
 	struct impl *this = data;
 	struct spa_event *event;
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct spa_pod_frame f[1];
+	uint32_t n_volume = dev->format.channels;
+	float volume[n_volume];
+	float soft_volume[n_volume];
 
-	spa_log_info(this->log, "device %s soft volume %f", dev->name, volume[0]);
+	spa_log_info(this->log, "device %s volume changed", dev->name);
+	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
+	this->params[IDX_Route].user++;
+
+	spa_zero(volume);
+	spa_zero(soft_volume);
+	acp_device_get_volume(dev, volume, n_volume);
+	acp_device_get_soft_volume(dev, soft_volume, n_volume);
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 	spa_pod_builder_push_object(&b, &f[0],
@@ -816,22 +825,28 @@ static void on_set_soft_volume(void *data, struct acp_device *dev,
 						SPA_TYPE_Float, n_volume, volume),
 			SPA_PROP_channelMap, SPA_POD_Array(sizeof(uint32_t),
 						SPA_TYPE_Id, dev->format.channels,
-						dev->format.map));
+						dev->format.map),
+			SPA_PROP_softVolumes, SPA_POD_Array(sizeof(float),
+						SPA_TYPE_Float, n_volume, soft_volume));
 	event = spa_pod_builder_pop(&b, &f[0]);
 
 	spa_device_emit_event(&this->hooks, event);
 }
 
-static void on_set_soft_mute(void *data, struct acp_device *dev,
-		bool mute)
+static void on_mute_changed(void *data, struct acp_device *dev)
 {
 	struct impl *this = data;
 	struct spa_event *event;
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct spa_pod_frame f[1];
+	bool mute;
 
-	spa_log_info(this->log, "device %s soft mute %d", dev->name, mute);
+	spa_log_info(this->log, "device %s mute changed", dev->name);
+	this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
+	this->params[IDX_Route].user++;
+
+	acp_device_get_mute(dev, &mute);
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 	spa_pod_builder_push_object(&b, &f[0],
@@ -842,7 +857,8 @@ static void on_set_soft_mute(void *data, struct acp_device *dev,
 
 	spa_pod_builder_add_object(&b,
 			SPA_TYPE_OBJECT_Props, SPA_EVENT_DEVICE_Props,
-			SPA_PROP_mute, SPA_POD_Bool(mute));
+			SPA_PROP_mute, SPA_POD_Bool(mute),
+			SPA_PROP_softMute, SPA_POD_Bool(mute));
 	event = spa_pod_builder_pop(&b, &f[0]);
 
 	spa_device_emit_event(&this->hooks, event);
@@ -857,8 +873,6 @@ struct acp_card_events card_events = {
 	.port_available = card_port_available,
 	.volume_changed = on_volume_changed,
 	.mute_changed = on_mute_changed,
-	.set_soft_volume = on_set_soft_volume,
-	.set_soft_mute = on_set_soft_mute,
 };
 
 static int impl_get_interface(struct spa_handle *handle, const char *type, void **interface)
@@ -870,7 +884,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Device) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Device))
 		*interface = &this->device;
 	else
 		return -ENOENT;
@@ -927,6 +941,7 @@ impl_init(const struct spa_handle_factory *factory,
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
 	this->loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Loop);
+	acp_i18n = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_I18N);
 	if (this->loop == NULL) {
 		spa_log_error(this->log, "a Loop interface is needed");
 		return -EINVAL;
@@ -947,9 +962,9 @@ impl_init(const struct spa_handle_factory *factory,
 		if ((str = spa_dict_lookup(info, SPA_KEY_API_ALSA_PATH)) != NULL)
 			snprintf(this->props.device, sizeof(this->props.device), "%s", str);
 		if ((str = spa_dict_lookup(info, "api.acp.auto-port")) != NULL)
-			this->props.auto_port = strcmp(str, "true") == 0 || atoi(str) != 0;
+			this->props.auto_port = spa_atob(str);
 		if ((str = spa_dict_lookup(info, "api.acp.auto-profile")) != NULL)
-			this->props.auto_profile = strcmp(str, "true") == 0 || atoi(str) != 0;
+			this->props.auto_profile = spa_atob(str);
 
 		items = alloca((info->n_items) * sizeof(*items));
 		spa_dict_for_each(it, info)
