@@ -33,17 +33,20 @@ extern "C" {
 #include <math.h>
 
 #include <alsa/asoundlib.h>
+#include <alsa/use-case.h>
 
 #include <spa/support/plugin.h>
 #include <spa/support/loop.h>
 #include <spa/support/log.h>
 #include <spa/utils/list.h>
+#include <spa/utils/json.h>
 
 #include <spa/node/node.h>
 #include <spa/node/utils.h>
 #include <spa/node/io.h>
 #include <spa/debug/types.h>
 #include <spa/param/param.h>
+#include <spa/param/latency-utils.h>
 #include <spa/param/audio/format-utils.h>
 
 #include "dll.h"
@@ -92,6 +95,7 @@ struct state {
 	struct spa_system *data_system;
 	struct spa_loop *data_loop;
 
+	int card_index;
 	snd_pcm_stream_t stream;
 	snd_output_t *output;
 
@@ -100,7 +104,11 @@ struct state {
 
 	uint64_t info_all;
 	struct spa_node_info info;
-	struct spa_param_info params[8];
+#define IDX_PropInfo	0
+#define IDX_Props	1
+#define IDX_NODE_IO	2
+#define N_NODE_PARAMS	3
+	struct spa_param_info params[N_NODE_PARAMS];
 	struct props props;
 
 	bool opened;
@@ -112,6 +120,7 @@ struct state {
 
 	uint32_t default_period_size;
 	uint32_t default_headroom;
+	uint32_t default_start_delay;
 	uint32_t default_format;
 	unsigned int default_channels;
 	unsigned int default_rate;
@@ -132,7 +141,14 @@ struct state {
 
 	uint64_t port_info_all;
 	struct spa_port_info port_info;
-	struct spa_param_info port_params[8];
+#define IDX_EnumFormat	0
+#define IDX_Meta	1
+#define IDX_IO		2
+#define IDX_Format	3
+#define IDX_Buffers	4
+#define IDX_Latency	5
+#define N_PORT_PARAMS	6
+	struct spa_param_info port_params[N_PORT_PARAMS];
 	struct spa_io_buffers *io;
 	struct spa_io_clock *clock;
 	struct spa_io_position *position;
@@ -152,6 +168,7 @@ struct state {
 	uint32_t threshold;
 	uint32_t last_threshold;
 	uint32_t headroom;
+	uint32_t start_delay;
 
 	uint32_t duration;
 	uint32_t last_duration;
@@ -164,6 +181,8 @@ struct state {
 	unsigned int resample:1;
 	unsigned int use_mmap:1;
 	unsigned int planar:1;
+	unsigned int freewheel:1;
+	unsigned int open_ucm:1;
 
 	int64_t sample_count;
 
@@ -176,6 +195,10 @@ struct state {
 
 	struct spa_dll dll;
 	double max_error;
+
+	struct spa_latency_info latency;
+
+	snd_use_case_mgr_t *ucm;
 };
 
 int
@@ -185,6 +208,9 @@ spa_alsa_enum_format(struct state *state, int seq,
 
 int spa_alsa_set_format(struct state *state, struct spa_audio_info *info, uint32_t flags);
 
+int spa_alsa_init(struct state *state);
+int spa_alsa_clear(struct state *state);
+
 int spa_alsa_open(struct state *state);
 int spa_alsa_start(struct state *state);
 int spa_alsa_reassign_follower(struct state *state);
@@ -192,7 +218,8 @@ int spa_alsa_pause(struct state *state);
 int spa_alsa_close(struct state *state);
 
 int spa_alsa_write(struct state *state);
-int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence);
+int spa_alsa_read(struct state *state);
+int spa_alsa_skip(struct state *state);
 
 void spa_alsa_recycle_buffer(struct state *state, uint32_t buffer_id);
 
@@ -206,16 +233,32 @@ static inline uint32_t spa_alsa_format_from_name(const char *name, size_t len)
 	return SPA_AUDIO_FORMAT_UNKNOWN;
 }
 
-static inline uint32_t spa_alsa_channel_from_name(const char *name, size_t len)
+static inline uint32_t spa_alsa_channel_from_name(const char *name)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (strncmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name), len) == 0)
+		if (strcmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)) == 0)
 			return spa_type_audio_channel[i].type;
 	}
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
 }
 
+static inline void spa_alsa_parse_position(struct channel_map *map, const char *val, size_t len)
+{
+	struct spa_json it[2];
+	char v[256];
+
+	spa_json_init(&it[0], val, len);
+        if (spa_json_enter_array(&it[0], &it[1]) <= 0)
+                spa_json_init(&it[1], val, len);
+
+	map->channels = 0;
+	while (spa_json_get_string(&it[1], v, sizeof(v)) > 0 &&
+	    map->channels < SPA_AUDIO_MAX_CHANNELS) {
+		map->pos[map->channels++] = spa_alsa_channel_from_name(v);
+	}
+}
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
