@@ -32,6 +32,7 @@
 #include <spa/utils/result.h>
 #include <spa/utils/list.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/node/node.h>
 #include <spa/buffer/alloc.h>
 #include <spa/node/io.h>
@@ -127,6 +128,7 @@ struct impl {
 static void emit_node_info(struct impl *this, bool full)
 {
 	uint32_t i;
+	uint64_t old = full ? this->info.change_mask : 0;
 
 	if (this->add_listener)
 		return;
@@ -143,7 +145,7 @@ static void emit_node_info(struct impl *this, bool full)
 			}
 		}
 		spa_node_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
@@ -1039,11 +1041,19 @@ impl_node_port_set_param(void *object,
 	spa_log_debug(this->log, NAME " %p: set param %u on port %d:%d %p",
 				this, id, direction, port_id, param);
 
-	is_monitor = IS_MONITOR_PORT(this, direction, port_id);
-	if (is_monitor)
-		target = this->fmt[SPA_DIRECTION_INPUT];
-	else
-		target = this->fmt[direction];
+	switch (id) {
+	case SPA_PARAM_Latency:
+		target = this->fmt[SPA_DIRECTION_REVERSE(direction)];
+		port_id = 0;
+		break;
+	default:
+		is_monitor = IS_MONITOR_PORT(this, direction, port_id);
+		if (is_monitor)
+			target = this->fmt[SPA_DIRECTION_INPUT];
+		else
+			target = this->fmt[direction];
+		break;
+	}
 
 	if ((res = spa_node_port_set_param(target,
 					direction, port_id, id, flags, param)) < 0)
@@ -1190,7 +1200,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Node))
 		*interface = &this->node;
 	else
 		return -ENOENT;
@@ -1288,37 +1298,37 @@ impl_init(const struct spa_handle_factory *factory,
 	this->info.params = this->params;
 	this->info.n_params = 4;
 
-	this->hnd_merger = SPA_MEMBER(this, sizeof(struct impl), struct spa_handle);
+	this->hnd_merger = SPA_PTROFF(this, sizeof(struct impl), struct spa_handle);
 	spa_handle_factory_init(&spa_merger_factory,
 				this->hnd_merger,
 				info, support, n_support);
 	size = spa_handle_factory_get_size(&spa_merger_factory, info);
 
-	this->hnd_convert_in = SPA_MEMBER(this->hnd_merger, size, struct spa_handle);
+	this->hnd_convert_in = SPA_PTROFF(this->hnd_merger, size, struct spa_handle);
 	spa_handle_factory_init(&spa_fmtconvert_factory,
 				this->hnd_convert_in,
 				info, support, n_support);
 	size = spa_handle_factory_get_size(&spa_fmtconvert_factory, info);
 
-	this->hnd_channelmix = SPA_MEMBER(this->hnd_convert_in, size, struct spa_handle);
+	this->hnd_channelmix = SPA_PTROFF(this->hnd_convert_in, size, struct spa_handle);
 	spa_handle_factory_init(&spa_channelmix_factory,
 				this->hnd_channelmix,
 				info, support, n_support);
 	size = spa_handle_factory_get_size(&spa_channelmix_factory, info);
 
-	this->hnd_resample = SPA_MEMBER(this->hnd_channelmix, size, struct spa_handle);
+	this->hnd_resample = SPA_PTROFF(this->hnd_channelmix, size, struct spa_handle);
 	spa_handle_factory_init(&spa_resample_factory,
 				this->hnd_resample,
 				info, support, n_support);
 	size = spa_handle_factory_get_size(&spa_resample_factory, info);
 
-	this->hnd_convert_out = SPA_MEMBER(this->hnd_resample, size, struct spa_handle);
+	this->hnd_convert_out = SPA_PTROFF(this->hnd_resample, size, struct spa_handle);
 	spa_handle_factory_init(&spa_fmtconvert_factory,
 				this->hnd_convert_out,
 				info, support, n_support);
 	size = spa_handle_factory_get_size(&spa_fmtconvert_factory, info);
 
-	this->hnd_splitter = SPA_MEMBER(this->hnd_convert_out, size, struct spa_handle);
+	this->hnd_splitter = SPA_PTROFF(this->hnd_convert_out, size, struct spa_handle);
 	spa_handle_factory_init(&spa_splitter_factory,
 				this->hnd_splitter,
 				info, support, n_support);
