@@ -34,6 +34,7 @@
 #include <spa/utils/type.h>
 #include <spa/utils/keys.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/node/node.h>
 #include <spa/support/loop.h>
 #include <spa/support/plugin.h>
@@ -154,8 +155,10 @@ static void init_node(struct impl *this, struct node *node, uint32_t id)
 
 	spa_zero(*node);
 	node->id = id;
-	for (i = 0; i < SPA_AUDIO_MAX_CHANNELS; i++)
-		node->volumes[i] = 1.0;
+	for (i = 0; i < SPA_AUDIO_MAX_CHANNELS; i++) {
+		node->volumes[i] = 1.0f;
+		node->soft_volumes[i] = 1.0f;
+	}
 }
 
 static const struct a2dp_codec *get_a2dp_codec(enum spa_bluetooth_audio_codec id)
@@ -237,7 +240,7 @@ static void transport_destroy(void *userdata)
 	node->transport = NULL;
 }
 
-static void emit_volume(struct impl *this, struct node *node)
+static void emit_node_props(struct impl *this, struct node *node, bool full)
 {
 	struct spa_event *event;
 	uint8_t buffer[4096];
@@ -258,11 +261,23 @@ static void emit_volume(struct impl *this, struct node *node)
 				SPA_TYPE_Float, node->n_channels, node->soft_volumes),
 			SPA_PROP_channelMap, SPA_POD_Array(sizeof(uint32_t),
 				SPA_TYPE_Id, node->n_channels, node->channels));
+	if (full) {
+		spa_pod_builder_add(&b,
+			SPA_PROP_mute, SPA_POD_Bool(node->mute),
+			SPA_PROP_softMute, SPA_POD_Bool(node->mute),
+			SPA_PROP_latencyOffsetNsec, SPA_POD_Long(node->latency_offset),
+			0);
+	}
 	event = spa_pod_builder_pop(&b, &f[0]);
 
 	spa_device_emit_event(&this->hooks, event);
 }
 
+static void emit_volume(struct impl *this, struct node *node)
+{
+	emit_node_props(this, node, false);
+}
+
 static void emit_info(struct impl *this, bool full);
 
 static float node_get_hw_volume(struct node *node)
@@ -312,14 +327,11 @@ static void volume_changed(void *userdata)
 
 	node_update_soft_volumes(node, t_volume->volume);
 
+	emit_volume(impl, node);
+
 	impl->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
 	impl->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 	emit_info(impl, false);
-
-	/* It sometimes flips volume to over 100% in pavucontrol slider
-	 * if volume is emitted before route info emitting while node
-	 * volumes are not identical to route volumes. Not sure why. */
-	emit_volume(impl, node);
 }
 
 static const struct spa_bt_transport_events transport_events = {
@@ -381,6 +393,8 @@ static void emit_node(struct impl *this, struct spa_bt_transport *t,
 			spa_hook_remove(&this->nodes[id].transport_listener);
 		this->nodes[id].transport = t;
 		spa_bt_transport_add_listener(t, &this->nodes[id].transport_listener, &transport_events, &this->nodes[id]);
+
+		emit_node_props(this, &this->nodes[id], true);
 	}
 }
 
@@ -602,13 +616,14 @@ static const struct spa_dict_item info_items[] = {
 
 static void emit_info(struct impl *this, bool full)
 {
+	uint64_t old = full ? this->info.change_mask : 0;
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
 		this->info.props = &SPA_DICT_INIT_ARRAY(info_items);
 
 		spa_device_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
@@ -631,8 +646,18 @@ static void emit_remove_nodes(struct impl *this)
 	}
 }
 
+static bool validate_profile(struct impl *this, uint32_t profile,
+		enum spa_bluetooth_audio_codec codec);
+
 static int set_profile(struct impl *this, uint32_t profile, enum spa_bluetooth_audio_codec codec)
 {
+	if (!validate_profile(this, profile, codec)) {
+		spa_log_warn(this->log, NAME ": trying to set invalid profile %d, codec %d, %08x %08x",
+			    profile, codec,
+			    this->bt_dev->profiles, this->bt_dev->connected_profiles);
+		return -EINVAL;
+	}
+
 	if (this->profile == profile &&
 	    (this->profile != DEVICE_PROFILE_A2DP || codec == this->props.codec) &&
 	    (this->profile != DEVICE_PROFILE_HSP_HFP || codec == this->props.codec))
@@ -970,7 +995,7 @@ static void set_initial_profile(struct impl *this)
 	const char *str;
 	if (this->bt_dev->settings != NULL) {
 		str = spa_dict_lookup(this->bt_dev->settings, "device.profile");
-		if (str != NULL && strcmp(str, "headset-head-unit") == 0 && find_hsp_hfp_profile(this))
+		if (str != NULL && spa_streq(str, "headset-head-unit") && find_hsp_hfp_profile(this))
 			return;
 	}
 
@@ -1125,6 +1150,16 @@ static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *
 	return spa_pod_builder_pop(b, &f[0]);
 }
 
+static bool validate_profile(struct impl *this, uint32_t profile,
+		enum spa_bluetooth_audio_codec codec)
+{
+	struct spa_pod_builder b = { 0 };
+	uint8_t buffer[1024];
+
+	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+	return (build_profile(this, &b, 0, 0, profile, codec) != NULL);
+}
+
 static struct spa_pod *build_route(struct impl *this, struct spa_pod_builder *b,
 		uint32_t id, uint32_t port, uint32_t dev, uint32_t profile)
 {
@@ -1236,17 +1271,12 @@ static struct spa_pod *build_route(struct impl *this, struct spa_pod_builder *b,
 	spa_pod_builder_prop(b, SPA_PARAM_ROUTE_profiles, 0);
 	spa_pod_builder_push_array(b, &f[1]);
 	for (i = 1; (j = get_profile_from_index(this, i, &next, &codec)) != SPA_ID_INVALID; i = next) {
-		struct spa_pod_builder b2 = { 0 };
-		uint8_t buffer[1024];
-		struct spa_pod *param;
 
 		if (!(profile_direction_mask(this, j) & (1 << direction)))
 			continue;
 
 		/* Check the profile actually exists */
-		spa_pod_builder_init(&b2, buffer, sizeof(buffer));
-		param = build_profile(this, &b2, 0, i, j, codec);
-		if (param == NULL)
+		if (!validate_profile(this, j, codec))
 			continue;
 
 		spa_pod_builder_int(b, i);
@@ -1518,7 +1548,7 @@ static int node_set_volume(struct impl *this, struct node *node, float volumes[]
 	if (n_volumes == 0)
 		return -EINVAL;
 
-	spa_log_debug(this->log, "node %p volume %f", node, volumes[0]);
+	spa_log_info(this->log, "node %p volume %f", node, volumes[0]);
 
 	for (i = 0; i < node->n_channels; i++) {
 		if (node->volumes[i] == volumes[i % n_volumes])
@@ -1541,6 +1571,8 @@ static int node_set_volume(struct impl *this, struct node *node, float volumes[]
 			node->soft_volumes[i] = node->volumes[i];
 	}
 
+	emit_volume(this, node);
+
 	return changed;
 }
 
@@ -1693,8 +1725,7 @@ static int impl_set_param(void *object,
 			return -EINVAL;
 
 		spa_log_debug(this->log, NAME": setting profile %d codec:%d", profile, codec);
-		set_profile(this, profile, codec);
-		break;
+		return set_profile(this, profile, codec);
 	}
 	case SPA_PARAM_Route:
 	{
@@ -1728,8 +1759,6 @@ static int impl_set_param(void *object,
 				this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
 			}
 			emit_info(this, false);
-			/* See volume_changed(void *) */
-			emit_volume(this, node);
 		}
 		break;
 	}
@@ -1755,19 +1784,16 @@ static int impl_set_param(void *object,
 			size_t j;
 			for (j = 0; j < this->supported_codec_count; ++j) {
 				if (this->supported_codecs[j]->id == codec_id) {
-					set_profile(this, this->profile, codec_id);
-					return 0;
+					return set_profile(this, this->profile, codec_id);
 				}
 			}
 		} else if (this->profile == DEVICE_PROFILE_HSP_HFP) {
 			if (codec_id == SPA_BLUETOOTH_AUDIO_CODEC_CVSD &&
 					spa_bt_device_supports_hfp_codec(this->bt_dev, HFP_AUDIO_CODEC_CVSD) == 1) {
-				set_profile(this, this->profile, codec_id);
-				return 0;
+				return set_profile(this, this->profile, codec_id);
 			} else if (codec_id == SPA_BLUETOOTH_AUDIO_CODEC_MSBC &&
 					spa_bt_device_supports_hfp_codec(this->bt_dev, HFP_AUDIO_CODEC_MSBC) == 1) {
-				set_profile(this, this->profile, codec_id);
-				return 0;
+				return set_profile(this, this->profile, codec_id);
 			}
 		}
 		return -EINVAL;
@@ -1795,7 +1821,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Device) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Device))
 		*interface = &this->device;
 	else
 		return -ENOENT;
