@@ -1374,8 +1374,11 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 		else if (port->memtype == V4L2_MEMORY_DMABUF) {
 			b->v4l2_buffer.m.fd = d[0].fd;
 		}
-		else
+		else {
+			spa_log_error(this->log, "v4l2: invalid port memory %d",
+					port->memtype);
 			return -EIO;
+		}
 
 		spa_v4l2_buffer_recycle(this, i);
 	}
@@ -1457,7 +1460,9 @@ mmap_init(struct impl *this,
 
 		spa_log_debug(this->log, "v4l2: data types %08x", d[0].type);
 
-		if (port->have_expbuf && (d[0].type & (1u << SPA_DATA_DmaBuf))) {
+		if (port->have_expbuf &&
+		    d[0].type != SPA_ID_INVALID &&
+		    (d[0].type & (1u << SPA_DATA_DmaBuf))) {
 			struct v4l2_exportbuffer expbuf;
 
 			spa_zero(expbuf);
@@ -1481,11 +1486,11 @@ mmap_init(struct impl *this,
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_ALLOCATED);
 			spa_log_debug(this->log, "v4l2: EXPBUF fd:%d", expbuf.fd);
 			use_expbuf = true;
-		} else {
+		} else if (d[0].type & (1u << SPA_DATA_MemPtr)) {
 fallback:
-			d[0].type = SPA_DATA_MemFd;
+			d[0].type = SPA_DATA_MemPtr;
 			d[0].flags = SPA_DATA_FLAG_READABLE;
-			d[0].fd = dev->fd;
+			d[0].fd = -1;
 			d[0].mapoffset = b->v4l2_buffer.m.offset;
 			d[0].data = mmap(NULL,
 					b->v4l2_buffer.length,
@@ -1500,6 +1505,9 @@ fallback:
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 			spa_log_debug(this->log, "v4l2: mmap offset:%u data:%p", d[0].mapoffset, b->ptr);
 			use_expbuf = false;
+		} else {
+			spa_log_error(this->log, "v4l2: unsupported data type:%08x", d[0].type);
+			return -ENOTSUP;
 		}
 		spa_v4l2_buffer_recycle(this, i);
 	}
@@ -1540,8 +1548,11 @@ spa_v4l2_alloc_buffers(struct impl *this,
 	} else if (dev->cap.capabilities & V4L2_CAP_READWRITE) {
 		if ((res = read_init(this)) < 0)
 			return res;
-	} else
+	} else {
+		spa_log_error(this->log, "v4l2: invalid capabilities %08x",
+					dev->cap.capabilities);
 		return -EIO;
+	}
 
 	return 0;
 }
