@@ -24,6 +24,8 @@
 
 #define EXT_STREAM_RESTORE_VERSION	1
 
+#include <spa/utils/string.h>
+
 static const struct extension_sub ext_stream_restore[];
 
 static int do_extension_stream_restore_test(struct client *client, uint32_t command, uint32_t tag, struct message *m)
@@ -139,15 +141,15 @@ static int do_extension_stream_restore_read(struct client *client, uint32_t comm
 			continue;
 
 		while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
-			if (strcmp(key, "volume") == 0) {
+			if (spa_streq(key, "volume")) {
 				if (spa_json_get_float(&it[1], &volume) <= 0)
 					continue;
 			}
-			else if (strcmp(key, "mute") == 0) {
+			else if (spa_streq(key, "mute")) {
 				if (spa_json_get_bool(&it[1], &mute) <= 0)
 					continue;
 			}
-			else if (strcmp(key, "volumes") == 0) {
+			else if (spa_streq(key, "volumes")) {
 				vol = VOLUME_INIT;
 				if (spa_json_enter_array(&it[1], &it[2]) <= 0)
 					continue;
@@ -157,7 +159,7 @@ static int do_extension_stream_restore_read(struct client *client, uint32_t comm
 						break;
 				}
 			}
-			else if (strcmp(key, "channels") == 0) {
+			else if (spa_streq(key, "channels")) {
 				if (spa_json_enter_array(&it[1], &it[2]) <= 0)
 					continue;
 
@@ -168,7 +170,7 @@ static int do_extension_stream_restore_read(struct client *client, uint32_t comm
 					map.map[map.channels] = channel_name2id(chname);
 				}
 			}
-			else if (strcmp(key, "target-node") == 0) {
+			else if (spa_streq(key, "target-node")) {
 				if (spa_json_get_string(&it[1], device_name, sizeof(device_name)) <= 0)
 					continue;
 			}
@@ -212,13 +214,14 @@ static int do_extension_stream_restore_write(struct client *client, uint32_t com
 		spa_zero(map);
 		spa_zero(vol);
 
-		message_get(m,
-			TAG_STRING, &name,
-			TAG_CHANNEL_MAP, &map,
-			TAG_CVOLUME, &vol,
-			TAG_STRING, &device_name,
-			TAG_BOOLEAN, &mute,
-			TAG_INVALID);
+		if (message_get(m,
+				TAG_STRING, &name,
+				TAG_CHANNEL_MAP, &map,
+				TAG_CVOLUME, &vol,
+				TAG_STRING, &device_name,
+				TAG_BOOLEAN, &mute,
+				TAG_INVALID) < 0)
+			return -EPROTO;
 
 		if (name == NULL || name[0] == '\0')
 			return -EPROTO;
@@ -239,17 +242,18 @@ static int do_extension_stream_restore_write(struct client *client, uint32_t com
 			fprintf(f, " ]");
 		}
 		if (device_name != NULL && device_name[0] &&
-		    (client->default_source == NULL || strcmp(device_name, client->default_source) != 0) &&
-		    (client->default_sink == NULL || strcmp(device_name, client->default_sink) != 0))
+		    (client->default_source == NULL || !spa_streq(device_name, client->default_source)) &&
+		    (client->default_sink == NULL || !spa_streq(device_name, client->default_sink)))
 			fprintf(f, ", \"target-node\": \"%s\"", device_name);
 		fprintf(f, " }");
 		fclose(f);
 
 		if (key_from_name(name, key, sizeof(key)) >= 0) {
 			pw_log_debug("%s -> %s: %s", name, key, ptr);
-			pw_manager_set_metadata(client->manager,
-					client->metadata_routes,
-					PW_ID_CORE, key, "Spa:String:JSON", "%s", ptr);
+			if (pw_manager_set_metadata(client->manager,
+							client->metadata_routes,
+							PW_ID_CORE, key, "Spa:String:JSON", "%s", ptr) < 0)
+				pw_log_warn(NAME ": failed to set metadata %s = %s", key, ptr);
 		}
 		free(ptr);
 	}
