@@ -26,6 +26,7 @@
 
 #include <unistd.h>
 #include <errno.h>
+#include <fcntl.h>
 #ifndef ENODATA
 #define ENODATA 9919
 #endif
@@ -37,7 +38,6 @@
 #ifndef HAVE_GETRANDOM
 # ifdef __FreeBSD__
 #  include <sys/param.h>
-#  include <fcntl.h>
 // FreeBSD versions < 12 do not have getrandom() syscall
 // Give a poor-man implementation here
 // Can be removed after September 30, 2021
@@ -45,6 +45,7 @@
 #   define GETRANDOM_FALLBACK	1
 #  endif
 # else
+#  include <fcntl.h>
 #  define GETRANDOM_FALLBACK	1
 # endif
 #endif
@@ -60,6 +61,7 @@ ssize_t getrandom(void *buf, size_t buflen, unsigned int flags) {
 }
 #endif
 
+#include <spa/utils/string.h>
 #include <spa/debug/types.h>
 
 #include "pipewire/impl.h"
@@ -92,7 +94,7 @@ static void * registry_bind(void *object, uint32_t id,
 	if (!PW_PERM_IS_R(permissions))
 		goto error_no_id;
 
-	if (strcmp(global->type, type) != 0)
+	if (!spa_streq(global->type, type))
 		goto error_wrong_interface;
 
 	pw_log_debug("global %p: bind global id %d, iface %s/%d to %d", global, id,
@@ -338,7 +340,7 @@ core_create_object(void *object,
 	if (!PW_PERM_IS_R(pw_global_get_permissions(factory->global, client)))
 		goto error_no_factory;
 
-	if (strcmp(factory->info.type, type) != 0)
+	if (!spa_streq(factory->info.type, type))
 		goto error_type;
 
 	if (factory->info.version < version)
@@ -454,15 +456,15 @@ struct pw_impl_core *pw_context_create_core(struct pw_context *context,
 	spa_hook_list_init(&this->listener_list);
 
 	if (user_data_size > 0)
-		this->user_data = SPA_MEMBER(this, sizeof(*this), void);
+		this->user_data = SPA_PTROFF(this, sizeof(*this), void);
 
 	pw_log_debug(NAME" %p: new %s", this, name);
 
 	return this;
 
 error_exit:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
+	free(this);
 	errno = -res;
 	return NULL;
 }
@@ -649,8 +651,7 @@ error_existed:
 	res = -EEXIST;
 	goto error_exit;
 error_exit:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
 	return res;
 }
 
