@@ -29,6 +29,8 @@
 #include <fcntl.h>
 #include <math.h>
 
+#include <spa/utils/string.h>
+
 #include "midifile.h"
 
 #define DEFAULT_TEMPO	500000	/* 500ms per quarter note (120 BPM) is the default */
@@ -263,10 +265,10 @@ midi_file_open(const char *filename, const char *mode, struct midi_file_info *in
 	if (mf == NULL)
 		return NULL;
 
-	if (strcmp(mode, "r") == 0) {
+	if (spa_streq(mode, "r")) {
 		if ((res = open_read(mf, filename, info)) < 0)
 			goto exit_free;
-	} else if (strcmp(mode, "w") == 0) {
+	} else if (spa_streq(mode, "w")) {
 		if ((res = open_write(mf, filename, info)) < 0)
 			goto exit_free;
 	} else {
@@ -545,7 +547,7 @@ static const char *program_names[] = {
 	"Celesta", "Glockenspiel", "Music Box", "Vibraphone", "Marimba",
 	"Xylophone", "Tubular Bells", "Dulcimer", "Drawbar Organ", "Percussive Organ",
 	"Rock Organ", "Church Organ", "Reed Organ", "Accoridan", "Harmonica",
-	"Tango Accordian", "Nylon String Guitar", "Steel String Guitar",
+	"Tango Accordion", "Nylon String Guitar", "Steel String Guitar",
 	"Electric Jazz Guitar", "Electric Clean Guitar", "Electric Muted Guitar",
 	"Overdriven Guitar", "Distortion Guitar", "Guitar Harmonics",
 	"Acoustic Bass", "Electric Bass (fingered)", "Electric Bass (picked)",
